//----------------------------------------------------------------------------------------------

// OpenClipDefines.h

//----------------------------------------------------------------------------------------------
/*
  OpenClip is an open-standard/no-copyright/nobody-owns-it format to store basic vector graphics
  in an XML format that can be used on any platform.
  
  Coordinate system is the same as PostScript: 72 coordinate units per inch for the image as
  printed on paper.  Origin is at the lower left, positive Y is up, positive X is right.
  Angles are in degrees, counterclockwise from 3:00.
  
  DrawingScale is the denomenator in the drawing scale, so would be 2 for a scale of 1/2.
  
  GlobalOrigin is a XY point to be used when there is a global origin.  Right now, it's just
  provided to be used.  We're not really sure how this would work, or even if it's needed here.
  
  InsertionPoint is a XY point to be used when clipboard objects are to be placed at a location
  such as the mouse point, for a Paste at Mouse operation.  
  
  Object information is intentionally simple and limited to basic drawing geometry, so that
  leaves out group structures and shadows.
  
  Basic object primitives are points, lines, rectangles, ellipses, polygons and Beziers. Rectangles
  and ellipses can have a rotation angle.  Rectangles can have a corner radius for rounded rectangles.
  Ellipses can have a start angle and arc angles to be arcs.  The angles are for a circle, even if
  the enclosing rectangle has a different height and width, and are for an unrotated shape, thus the
  rotation angle must be added to the start angle.  Angles may be positive or negative values.
  
  Rectangles are defined by two points at diagonal corners. When reading these in always rectify
  because you can never be sure which corners you are getting.
  
  When polygons and beziers are closed, the last point should be the same as the first point.
  
  Beziers are defined by segments, where the points A,B,C,D define a segment with A and D being node
  points and B and C being control handles.  Point D is the first point of the next segment, so the
  order of the points would be ABCD BCD BCD ... BCD.  
  
  The basic intent here is to keep the design very simple so it can be easily implemented, and
  it's intended to break down the barriers between 2D and 3D programs
  
  All objects can have 3D points.  Use SetObjectZ/OpenClip_Z for Z heights of 2D objects.
  
  AutoCAD objects have layer names for each object.  Use LayerName for each drawing object if you like.
  
  There is no provision for text at this time, and we are uncertain whether to attempt to handle
  this.
  
  Advanced is for a list of drawing objects with types beyond the points..Bezier range.  Add an
  advanced object, such as a Bspline or Nurbs, to a basic object.  In many cases, more advanced 
  programs have to provide a crude or down-graded representation of an object as a polygon or Bezier
  that any program can read, but this provides the ability to still include the more advanced,
  higher forms of graphic representations for other advanced programs to read.
  
  3DFace objects are the same as DXF 3DFace, 3 or 4 3D points
  
  --- Uses for OpenClip ---
  
  Put on Pasteboard to implement Copy and Paste for software that support OpenClip.
  
  Create Copy to OpenClip and Paste from OpenClip commands as plug-ins for each program, reading and
  writing from the following directory and file
  
      /Library/Application Support/OpenClip/OpenClip
  
  Because routines are available to read and write from URL, you can create "Far Copy" and "Far Paste"
  commands that read and write from a common URL, (which could be on a file server or on a website like
  a dot-Mac) allowing users to copy in one office on one machine, and paste on another machine, Mac to
  PC.  While clever and easy to do, this also is a potential technical support nightmare for anyone
  asked to support the users.
  
  OpenClip files could be used to import and export drawing files, but because OpenClip doesn't include
  text, bitmaps and dimension objects, it is questionable whether this makes sense.
  
*/
//----------------------------------------------------------------------------------------------


#ifndef __OpenClip_Keys__
#define __OpenClip_Keys__


// Top level objects

#define	OpenClip_DrawingScale		  @"DrawingScale"
#define	OpenClip_GlobalOrigin		  @"GlobalOrigin"
#define	OpenClip_InsertionPoint	  @"InsertionPoint"

// arrays for drawing objects, where each object is a dictionary describing the object.
#define	OpenClip_DrawingLayers	  @"DrawingLayers"
#define	OpenClip_DrawingObjects	  @"DrawingObjects"
#define	OpenClip_Arrows	          @"Arrows"
#define	OpenClip_ArrowAtStart     @"ArrowAtStart"
#define	OpenClip_ArrowAtEnd       @"ArrowAtEnd"
#define	OpenClip_Advanced	        @"Advanced"

#define	OpenClip_LayerName        @"LayerName"  // use for any drawing object, if you like
#define	OpenClip_LayerColor	      @"LayerColor"
#define	OpenClip_LayerScale	      @"LayerScale"

#define	OpenClip_ObjectType       @"ObjectType" // every object should have a type

                                                // constants for Basic ObjectType
#define	OpenClip_NullObj          @"NullObj"
#define	OpenClip_PointObj         @"PointsObj"
#define	OpenClip_LineObj          @"LineObj"
#define	OpenClip_RectObj          @"RectObj"
#define	OpenClip_EllipseObj       @"EllipseObj"
#define	OpenClip_PolygonObj       @"PolygonObj"
#define	OpenClip_BezierObj        @"BezierObj"
#define	OpenClip_TextObj          @"TextObj"    // reserved but not implemented
#define	OpenClip_GroupObj         @"GroupObj"   // has DrawingObjects array

                                                // constants for Advanced ObjectType
#define	OpenClip_BSplineObj       @"BSplineObj"

#define	OpenClip_Points		        @"Points"			// An array of dictionaries containing floating point values 
                                                // for (x,y,z,...) as required
#define	OpenClip_X	              @"x"          // use for Points, GlobalOrigin, InsertionPoint
#define	OpenClip_Y	              @"y"          // use for Points, GlobalOrigin, InsertionPoint
#define	OpenClip_Z	              @"z"          // use for Points, GlobalOrigin, InsertionPoint
                                                // and optionally for a Z height if you want to give a 2D
                                                // drawing object a Z height

#define	OpenClip_Closed		        @"Closed"			// Optional boolean. Use for polygons and Beziers.

#define	OpenClip_FillColor	      @"FillColor"	// Optional RGB color.  Assume not filled if no color given
#define	OpenClip_PenColor	        @"PenColor"		// Optional RGB color.
#define	OpenClip_Red	            @"r"
#define	OpenClip_Green	          @"g"
#define	OpenClip_Blue	            @"b"
#define	OpenClip_Alpha	          @"a"

#define	OpenClip_PenSize	        @"PenSize"    // Optional floating point value

#define	OpenClip_RotationAngle	  @"RotationAngle"  // for rectangles, ellipses
#define	OpenClip_StartAngle	      @"StartAngle"     // for ellipses, to make an arc
#define	OpenClip_ArcAngle	        @"ArcAngle"       // for ellipses, to make an arc

#define	OpenClip_CornerRadius     @"CornerRadius"   // for rounded rectangles

#define	OpenClip_LineDash         @"LineDash"       // for dashed lines

#define	OpenClip_Arrows           @"Arrows"         // for arrows at both ends
#define	OpenClip_ArrowAtStart     @"ArrowAtStart"
#define	OpenClip_ArrowAtEnd       @"ArrowAtEnd"

#define	OpenClip_Comment          @"Comment"  // for strings
#define	OpenClip_Note             @"Note"     // for strings

#define	OpenClip_Comments         @"Comments" // array of Comment
#define	OpenClip_Notes            @"Notes"    // array of Note

#define	OpenClip_Creator          @"Creator"
#define	OpenClip_WildToolsInfo    @"OpenClip for PowerCADD, see http://www.seqair.com/WildTools/OpenClip/"

#define	OpenClip_PasteboardFlavorType	 @"com.openclip.clip"

#endif // __OpenClip_Keys__
